<?php
class Netresearch_OPS_Test_Model_ObserverTest extends EcomDev_PHPUnit_Test_Case_Controller
{
    private $_model;
 
    public function setUp()
    {
        parent::setup();
        $this->_model = Mage::getModel('ops/observer');
    }
    
    public function testType()
    {
        $this->assertInstanceOf('Netresearch_OPS_Model_Observer', $this->_model);
    }

    public function testIsCheckoutWithAliasOrDd()
    {
        if (version_compare(PHP_VERSION, '5.3.2') >= 0) {
            $class = new ReflectionClass('Netresearch_OPS_Model_Observer');
            $method = $class->getMethod('isCheckoutWithAliasOrDd');
            $method->setAccessible(true);
         
            $this->assertTrue($method->invokeArgs($this->_model,array('ops_cc')));
            $this->assertTrue($method->invokeArgs($this->_model,array('ops_directDebit')));
            $this->assertFalse($method->invokeArgs($this->_model,array('checkmo')));
        }
    }
    
    
    public function testPerformDirectLinkRequestWithUnknownResponse()
    {
        $quote = new Varien_Object();
        $payment = $this->getModelMock('sales/quote_payment', array('save'));
        $payment->expects($this->any())
              ->method('save')
              ->will($this->returnValue(null));
        $this->replaceByMock('model', 'sales/quote_payment', $payment);
        $quote->setPayment($payment);
        $response = null;
        $directLinkMock = $this->getModelMock('ops/api_directlink', array('performRequest'));
        $directLinkMock->expects($this->any())
            ->method('performRequest')
            ->will($this->returnValue($response));
        $this->replaceByMock('model', 'ops/api_directlink', $directLinkMock);
        $observer = Mage::getModel('ops/observer');
        $observer->performDirectLinkRequest($quote, array());
        $this->assertFalse($this->setExpectedException('PHPUnit_Framework_ExpectationFailedException'));
        $this->assertTrue(array_key_exists('ops_response', $quote->getPayment()->getAdditionalInformation()));
    }
    
    public function testPerformDirectLinkRequestWithInvalidResponse()
    {
        $quote = new Varien_Object();
        $payment = $this->getModelMock('sales/quote_payment', array('save'));
        $payment->expects($this->any())
              ->method('save')
              ->will($this->returnValue(null));
        $this->replaceByMock('model', 'sales/quote_payment', $payment);
        $quote->setPayment($payment);
        $response = '';
        $directLinkMock = $this->getModelMock('ops/api_directlink', array('performRequest'));
        $directLinkMock->expects($this->any())
            ->method('performRequest')
            ->will($this->returnValue($response));
        $this->replaceByMock('model', 'ops/api_directlink', $directLinkMock);
        $observer = Mage::getModel('ops/observer');
        $this->assertTrue($this->setExpectedException('PHPUnit_Framework_ExpectationFailedException'));
        $observer->performDirectLinkRequest($quote, array());
        $this->assertFalse(array_key_exists('ops_response', $quote->getPayment()->getAdditionalInformation()));
    }
    
    public function testPerformDirectLinkRequestWithValidResponse()
    {
        $quote = new Varien_Object();
        $payment = $this->getModelMock('sales/quote_payment', array('save'));
        $payment->expects($this->any())
              ->method('save')
              ->will($this->returnValue(null));
        $this->replaceByMock('model', 'sales/quote_payment', $payment);
        $quote->setPayment($payment);
        $response = array('STATUS' => Netresearch_OPS_Model_Payment_Abstract::OPS_AUTHORIZED);
        $directLinkMock = $this->getModelMock('ops/api_directlink', array('performRequest'));
        $directLinkMock->expects($this->any())
            ->method('performRequest')
            ->will($this->returnValue($response));
        $this->replaceByMock('model', 'ops/api_directlink', $directLinkMock);
        $observer = Mage::getModel('ops/observer');
        $this->assertFalse($this->setExpectedException('PHPUnit_Framework_ExpectationFailedException'));
        $observer->performDirectLinkRequest($quote, array());
        $this->assertTrue(array_key_exists('ops_response', $quote->getPayment()->getAdditionalInformation()));
    }
    
    public function testPerformDirectLinkRequestWithValidResponseButInvalidStatus()
    {
        $quote = new Varien_Object();
        $payment = $this->getModelMock('sales/quote_payment', array('save'));
        $payment->expects($this->any())
              ->method('save')
              ->will($this->returnValue(null));
        $this->replaceByMock('model', 'sales/quote_payment', $payment);
        $quote->setPayment($payment);
        $response = array('STATUS' => Netresearch_OPS_Model_Payment_Abstract::OPS_AUTH_REFUSED);
        $directLinkMock = $this->getModelMock('ops/api_directlink', array('performRequest'));
        $directLinkMock->expects($this->any())
            ->method('performRequest')
            ->will($this->returnValue($response));
        $this->replaceByMock('model', 'ops/api_directlink', $directLinkMock);
        $observer = Mage::getModel('ops/observer');
        $this->assertTrue($this->setExpectedException('PHPUnit_Framework_ExpectationFailedException'));
        $observer->performDirectLinkRequest($quote, array());
        $this->assertFalse(array_key_exists('ops_response', $quote->getPayment()->getAdditionalInformation()));
    }

    public function testAppendCheckBoxToRefundForm()
    {
        Mage::register('current_creditmemo', null);
        $transport = new Varien_Object();
        $transport->setHtml('Foo');
        $observer = Mage::getModel('ops/observer');
        $event = new Varien_Object();
        $event->setBlock('');
        $this->assertEquals('', $observer->appendCheckBoxToRefundForm($event));

        $order = new Varien_Object();
        $payment = new Varien_Object();
        $methodInstance = Mage::getModel('ops/payment_cc');
        $payment->setMethodInstance($methodInstance);
        $order->setPayment($payment);
        $invoice = new Varien_Object();
        $invoice->setTransactionId(1);
        $creditMemo = $this->getModelMock('sales/order_creditmemo', array('getOrder', 'getInvoice', 'canRefund', 'getOrderId'));
        $creditMemo->expects($this->any())
            ->method('getOrder')
            ->will($this->returnValue($order));
        $creditMemo->expects($this->any())
            ->method('getInvoice')
            ->will($this->returnValue($invoice));
        $creditMemo->expects($this->any())
            ->method('canRefund')
            ->will($this->returnValue(true));
        $creditMemo->expects($this->any())
            ->method('getOrderId')
            ->will($this->returnValue(1));
        Mage::register('current_creditmemo', $creditMemo);
        $block = Mage::app()->getLayout()->getBlockSingleton('adminhtml/sales_order_creditmemo_totals');
        $parentBlock = Mage::app()->getLayout()->getBlockSingleton('adminhtml/sales_order_creditmemo_create_items');
        $block->setParentBlock($parentBlock);

        $blockMock = $this->getBlockMock('ops/adminhtml_sales_order_creditmemo_totals_checkbox', array('renderView'));
        $blockMock->expects($this->once())
            ->method('renderView')
            ->will($this->returnValue('<b>checkbox</b>'));
        $this->replaceByMock('block', 'ops/adminhtml_sales_order_creditmemo_totals_checkbox', $blockMock);
        $event->setBlock($block);
        $event->setTransport($transport);
        $html = $observer->appendCheckBoxToRefundForm($event);
        $this->assertEquals('Foo<b>checkbox</b>', $html);
        $this->assertNotEquals('Bar<span>checkbox</span>', $html);
        
        Mage::unregister('current_creditmemo');
    }
    
    public function testShowWarningForClosedTransactions()
    {
        Mage::register('current_creditmemo', null);
        $transport = new Varien_Object();
        $transport->setHtml('Foo');
        $observer = Mage::getModel('ops/observer');
        $event = new Varien_Object();
        $event->setBlock('');
        $this->assertEquals('', $observer->showWarningForClosedTransactions($event));

        $order = new Varien_Object();
        $payment = new Varien_Object();
        $methodInstance = Mage::getModel('ops/payment_cc');
        $payment->setMethodInstance($methodInstance);
        $order->setPayment($payment);
        $invoice = new Varien_Object();
        $invoice->setTransactionId(1);
        $creditMemo = $this->getModelMock('sales/order_creditmemo', array('getOrder', 'getInvoice', 'canRefund', 'getOrderId'));
        $creditMemo->expects($this->any())
            ->method('getOrder')
            ->will($this->returnValue($order));
        $creditMemo->expects($this->any())
            ->method('getInvoice')
            ->will($this->returnValue($invoice));
        $creditMemo->expects($this->any())
            ->method('canRefund')
            ->will($this->returnValue(false));
        $creditMemo->expects($this->any())
            ->method('getOrderId')
            ->will($this->returnValue(1));
        Mage::register('current_creditmemo', $creditMemo);
        $block = Mage::app()->getLayout()->getBlockSingleton('adminhtml/sales_order_creditmemo_create');

        $blockMock = $this->getBlockMock('ops/adminhtml_sales_order_creditmemo_closedTransaction_warning', array('renderView'));
        $blockMock->expects($this->once())
            ->method('renderView')
            ->will($this->returnValue('<b>warning</b>'));
        $this->replaceByMock('block', 'ops/adminhtml_sales_order_creditmemo_closedTransaction_warning', $blockMock);
        $event->setBlock($block);
        $event->setTransport($transport);
        $html = $observer->showWarningForClosedTransactions($event);
        $this->assertEquals('<b>warning</b>Foo', $html);
        $this->assertNotEquals('Bar<span>warning</span>', $html);
        
        Mage::unregister('current_creditmemo');
    }
}

