<?php
class Netresearch_OPS_Test_Model_Payment_AbstractTest extends EcomDev_PHPUnit_Test_Case_Controller
{

    /**
     * @test
     */
    public function _getOrderDescription()
    {
        $items = array(
            new Varien_Object(array(
                'parent_item' => false,
                'name'       => 'abc'
            )),
            new Varien_Object(array(
                'parent_item' => true,
                'name'       => 'def'
            )),
            new Varien_Object(array(
                'parent_item' => false,
                'name'       => 'ghi'
            )),
            new Varien_Object(array(
                'parent_item' => false,
                'name'       => 'Dubbelwerkende cilinder Boring ø70 Stang ø40 3/8'
            )),
            new Varien_Object(array(
                'parent_item' => false,
                'name'       => '0123456789012345678901234567890123456789012xxxxxx'
            )),
        );

        $order = $this->getModelMock('sales/order', array('getAllItems'));
        $order->expects($this->any())
            ->method('getAllItems')
            ->will($this->returnValue($items));

        $result = Mage::getModel('ops/payment_abstract')->_getOrderDescription($order);
        $this->assertEquals(
            'abc, ghi, Dubbelwerkende cilinder Boring 70 Stang 40 3/8, 012345678901234567890123456789012345678901',
            $result
        );
    }

    /**
     * check if payment methods return correct BRAND and PM values
     *
     * @loadExpectation paymentMethods
     * @test
     */
    public function shouldReturnCorrectBrandAndPMValues()
    {
        $paymentMethods = array(
            'acceptgiro',
            //'bankTransfer',      -> to be tested separately
            'cashU',
            'cbcOnline',
            //'cc',                -> to be tested separately
            'centeaOnline',
            'cod',
            //'dexiaDirectNet',    -> seems not to supported anymore
            //'directDebit',       -> to be tested separately
            //'directEbanking',    -> to be tested separately
            'eDankort',
            'eps',
            'fortisPayButton',
            'giroPay',
            'iDeal',
            'ingHomePay',
            //'interSolve',        -> to be tested separately
            'kbcOnline',
            //'miniTix',           -> seems not to supported anymore
            'openInvoiceDe',
            'openInvoiceNl',
            'paypal',
            'paysafecard',
            'pingPing',
            'postFinanceEFinance',
            'tunz',
            'wallie',
        );
        foreach ($paymentMethods as $methodIdentifier) {
            $method = Mage::getModel('ops/payment_' . $methodIdentifier);
            $this->assertEquals(
                $this->expected('ops_' . $methodIdentifier)->getBrand(),
                $method->getOpsBrand(null)
            );
            $this->assertEquals(
                $this->expected('ops_' . $methodIdentifier)->getPm(),
                $method->getOpsCode()
            );
        }
    }

    /**
     * check if payment method BankTransfer returns correct BRAND and PM values
     *
     * @loadExpectation paymentMethods
     * @test
     */
    public function shouldReturnCorrectBrandAndPMValuesForBankTransfer()
    {
        $method  = Mage::getModel('ops/payment_bankTransfer');

        $payment = $this->getModelMock('sales/quote_payment', array('getId'));
        $payment->expects($this->any())
            ->method('getId')
            ->will($this->returnValue('1'));
        $this->replaceByMock('model', 'sales/quote_payment', $payment);
        $quote  = $this->getModelMock('sales/quote', array('getPayment', 'getQuoteId'));
        $quote->expects($this->any())
            ->method('getQuoteId')
            ->will($this->returnValue('321'));
        $quote->expects($this->any())
            ->method('getPayment')
            ->will($this->returnValue($payment));

        $session = Mage::getSingleton(
            'checkout/session',
            array(
                'last_real_order_id' => '123',
                'quote'              => $quote
            )
        );

        $method = Mage::getModel('ops/payment_bankTransfer');
        try {
            $method->assignData(array('country_id' => 'DE'));
        } catch (Mage_Core_Exception $e) {
            if ('Cannot retrieve the payment information object instance.' != $e->getMessage()) {
                throw $e;
            }
        }
        $this->assertEquals(
            $this->expected('ops_bankTransferDe')->getBrand(),
            $method->getOpsBrand(null)
        );
        $reflectedMethod = new ReflectionMethod($method, 'getOpsCode');
        $reflectedMethod->setAccessible(true);
        $this->assertEquals(
            $this->expected('ops_bankTransferDe')->getPm(),
            $reflectedMethod->invoke($method)
        );
    }

    /**
     * @test
     * @loadFixture ../../../../var/fixtures/orders.yaml
     */
    public function testCanCancelManually()
    {
        $opsAbstractPayment = new Netresearch_OPS_Model_Payment_Abstract();
        
        //Check for successful can cancel (pending_payment and payment status 0)
        $order = Mage::getModel("sales/order")->load(11);
        $this->assertTrue($opsAbstractPayment->canCancelManually($order));
        
        //Check for successful cancel (pending_payment and payment status null/not existing)
        $order = Mage::getModel("sales/order")->load(14);
        $this->assertTrue($opsAbstractPayment->canCancelManually($order));
        
        //Check for denied can cancel (pending_payment and payment status 5)
        $order = Mage::getModel("sales/order")->load(12);
        $this->assertFalse($opsAbstractPayment->canCancelManually($order));
        
        //Check for denied can cancel (processing and payment status 0)
        $order = Mage::getModel("sales/order")->load(13);
        $this->assertFalse($opsAbstractPayment->canCancelManually($order));
    }


    public function testGetCloseTransactionFromCreditMemoData()
    {
        $reflection_class = new ReflectionClass("Netresearch_OPS_Model_Payment_Abstract");

        //Then we need to get the method we wish to test and
        //make it accessible
        $method = $reflection_class->getMethod("getCloseTransactionFromCreditMemoData");
        $method->setAccessible(true);

        //We need to create an empty object to pass to
        //ReflectionMethod invoke method followed by our
        //test parameters
        $paymentModel = new Netresearch_OPS_Model_Payment_Abstract(null);

        $this->assertFalse($method->invoke($paymentModel, array()));
        $this->assertFalse($method->invoke($paymentModel, array('ops_close_transaction' => 'OFF')));
        $this->assertFalse($method->invoke($paymentModel, array('ops_close_transaction' => 'off')));
        $this->assertFalse($method->invoke($paymentModel, array('ops_close_transaction' => '')));
        $this->assertFalse($method->invoke($paymentModel, array('ops_close_transaction' => 1)));

        $this->assertTrue($method->invoke($paymentModel, array('ops_close_transaction' => 'on')));
        $this->assertTrue($method->invoke($paymentModel, array('ops_close_transaction' => 'ON')));
    }


    /**
     * @test
     * @loadFixture ../../../../var/fixtures/orders.yaml
     */
    public function testCanRefund()
    {
        $paymentModel = Mage::getModel('ops/payment_abstract');
        Mage::app()->getRequest()->setParam('order_id', 17);
        $this->assertFalse($paymentModel->canRefund());

        Mage::app()->getRequest()->setParam('order_id', 18);
        $this->assertTrue($paymentModel->canRefund());

        Mage::app()->getRequest()->setParam('order_id', 11);
        $helperMock = $this->getHelperMock('ops/directlink', array('hasPaymentTransactions'));
        $helperMock->expects($this->any())
            ->method('hasPaymentTransactions')
            ->will($this->returnValue(false));
        $this->replaceByMock('helper', 'ops/directlink', $helperMock);

        $this->assertTrue($paymentModel->canRefund());

        $helperMock = $this->getHelperMock('ops/directlink', array('hasPaymentTransactions'));
        $helperMock->expects($this->any())
            ->method('hasPaymentTransactions')
            ->will($this->returnValue(true));
        $this->replaceByMock('helper', 'ops/directlink', $helperMock);

        $helperMock = $this->getHelperMock('ops/data', array('redirect'));
        $this->replaceByMock('helper', 'ops/data', $helperMock);

        $paymentModel->canRefund();
        $messages = Mage::getSingleton('core/session')->getMessages();
        $messageItems = $messages->getItems();
        $message = array_pop($messageItems);
        $this->assertEquals(Mage::helper('ops/data')->__('There is already one creditmemo in the queue. The Creditmemo will be created automatically as soon as Ogone sends an acknowledgement.'), $message->getText());


        $helperMock = $this->getHelperMock('ops/directlink', array('hasPaymentTransactions'));
        $helperMock->expects($this->any())
            ->method('hasPaymentTransactions')
            ->will($this->throwException(new Exception('foo')));
        $this->replaceByMock('helper', 'ops/directlink', $helperMock);
        $this->assertTrue($paymentModel->canRefund());
    }


    public function testGetFormFieldsEmptyWithNonExistingOrder()
    {
        $paymentModel = Mage::getModel('ops/payment_abstract');
        $this->assertTrue(is_array($paymentModel->getFormFields(null, array())));
        $this->assertEquals(0, count($paymentModel->getFormFields(null, array())));
    }

    public function testGetFormFieldsWithEmptyOrderPassedButExistingOrder()
    {
        $order = new Varien_Object();
        $payment = new Varien_Object();
        $payment->setMethodInstance(Mage::getModel('ops/payment_cc'));
        $order->setPayment($payment);
        $paymentModel = $this->getModelMock('ops/payment_abstract', array('getMethodDependendFormFields', 'getOrder'));
        $paymentModel->expects($this->once())
            ->method('getOrder')
            ->will($this->returnValue($order));
        $formFields = $paymentModel->getFormFields(null, array());
        $this->assertArrayHasKey('PSPID', $formFields);
        $this->assertArrayHasKey('SHASIGN', $formFields);
    }

    public function testGetFormFields()
    {
        $paymentModel = $this->getModelMock('ops/payment_abstract', array('getMethodDependendFormFields'));
        $configMock = $this->getModelMock('ops/config', array('getPSPID'));
        $configMock->expects($this->once())
            ->method('getPSPID')
            ->with(null)
            ->will($this->returnValue('NRMAGENTO'));
        $this->replaceByMock('model', 'ops/config', $configMock);
        $helperMock = $this->getHelperMock('ops/payment', array('getShaSign'));
        $helperMock->expects($this->any())
            ->method('getSHASign')
            ->with(
                $this->anything(),
                $this->anything(),
                null
            )
            ->will($this->returnValue('SHA123'));
        $this->replaceByMock('helper', 'ops/payment', $helperMock);
        $order = new Varien_Object();
        $payment = new Varien_Object();
        $payment->setMethodInstance(Mage::getModel('ops/payment_cc'));
        $order->setPayment($payment);
        $formFields = $paymentModel->getFormFields($order, array());
        $this->assertArrayHasKey('PSPID', $formFields);
        $this->assertArrayHasKey('SHASIGN', $formFields);
        $this->assertEquals('NRMAGENTO', $formFields['PSPID']);
        $this->assertEquals('2d9f92d6f3955847ab2db427be75fe7eb0cde045', $formFields['SHASIGN']);
    }

    public function testGetFormFieldsWithTPParam()
    {
        $paymentModel = $this->getModelMock('ops/payment_abstract', array('getMethodDependendFormFields'));
        $configMock = $this->getModelMock('ops/config', array('getPSPID', 'getconfigData'));
        $configMock->expects($this->once())
            ->method('getPSPID')
            ->with(null)
            ->will($this->returnValue('NRMAGENTO'));
        $configMock->expects($this->any())
            ->method('getConfigData')
//            ->with('template')
            ->will($this->returnValue('spo'));
        $this->replaceByMock('model', 'ops/config', $configMock);
        $helperMock = $this->getHelperMock('ops/payment', array('getShaSign'));
        $helperMock->expects($this->any())
            ->method('getSHASign')
            ->with(
                $this->anything(),
                $this->anything(),
                null
            )
            ->will($this->returnValue('SHA123'));
        $this->replaceByMock('helper', 'ops/payment', $helperMock);
        $order = new Varien_Object();
        $payment = new Varien_Object();
        $payment->setMethodInstance(Mage::getModel('ops/payment_cc'));
        $order->setPayment($payment);
        $formFields = $paymentModel->getFormFields($order, array());
        $this->assertArrayHasKey('PSPID', $formFields);
        $this->assertArrayHasKey('SHASIGN', $formFields);
        $this->assertArrayHasKey('TP', $formFields);
        $this->assertEquals('NRMAGENTO', $formFields['PSPID']);
        $this->assertEquals('2d9f92d6f3955847ab2db427be75fe7eb0cde045', $formFields['SHASIGN']);
    }

    public function testGetFormFieldsWithFormDependendFormFields()
    {
        $paymentModel = $this->getModelMock('ops/payment_abstract', array('getMethodDependendFormFields'));
        $paymentModel->expects($this->any())
            ->method('getMethodDependendFormFields')
            ->will($this->returnValue(array('foo' => 'bla')));
        $configMock = $this->getModelMock('ops/config', array('getPSPID'));
        $configMock->expects($this->once())
            ->method('getPSPID')
            ->with(null)
            ->will($this->returnValue('NRMAGENTO'));
        $this->replaceByMock('model', 'ops/config', $configMock);
        $helperMock = $this->getHelperMock('ops/payment', array('getShaSign'));
        $helperMock->expects($this->any())
            ->method('getSHASign')
            ->with(
                $this->anything(),
                $this->anything(),
                null
            )
            ->will($this->returnValue('SHA123'));
        $this->replaceByMock('helper', 'ops/payment', $helperMock);
        $order = new Varien_Object();
        $payment = new Varien_Object();
        $payment->setMethodInstance(Mage::getModel('ops/payment_cc'));
        $order->setPayment($payment);
        $formFields = $paymentModel->getFormFields($order, array());
        $this->assertArrayHasKey('PSPID', $formFields);
        $this->assertArrayHasKey('SHASIGN', $formFields);
        $this->assertArrayHasKey('foo', $formFields);
        $this->assertEquals('NRMAGENTO', $formFields['PSPID']);
        $this->assertEquals('2d9f92d6f3955847ab2db427be75fe7eb0cde045', $formFields['SHASIGN']);
        $this->assertEquals('bla', $formFields['foo']);
    }

    public function testGetFormFieldsWithStoreId()
    {
        $paymentModel = $this->getModelMock('ops/payment_abstract', array('getMethodDependendFormFields'));
        $configMock = $this->getModelMock('ops/config', array('getPSPID', 'getSHASign'));
        $configMock->expects($this->once())
            ->method('getPSPID')
            ->with(1)
            ->will($this->returnValue('NRMAGENTO5'));
        $this->replaceByMock('model', 'ops/config', $configMock);

        $helperMock = $this->getHelperMock('ops/payment', array('getShaSign'));

        $helperMock->expects($this->any())
            ->method('getSHASign')
            ->with(
                $this->anything(),
                $this->anything(),
                1
            )
            ->will($this->returnValue('SHA987'));
        $this->replaceByMock('helper', 'ops/payment', $helperMock);
        $order = new Varien_Object();
        $payment = new Varien_Object();
        $order->setStoreId(1);
        $payment->setMethodInstance(Mage::getModel('ops/payment_cc'));
        $order->setPayment($payment);
        $formFields = $paymentModel->getFormFields($order, array());
        $this->assertArrayHasKey('PSPID', $formFields);
        $this->assertArrayHasKey('SHASIGN', $formFields);
        $this->assertEquals('NRMAGENTO5', $formFields['PSPID']);
        $this->assertEquals('0f119cdea2f8ddc0c852bab4765f12d3913982fc', $formFields['SHASIGN']);
    }
}
